#!/usr/bin/env python

############################################################################
#    Copyright (C) 2004 by                                                 #
#    Petteri Klemola (pzq at users dot sourceforge dot net)                #
#    Mauno Haukila (mauno at users dot sourceforge dot net)                #
#    Tero Kuusela (teroajk at users dot sourceforge dot net)               #
#    Stefan Brockmann (stefankjb at yahoo dot com)                         #
#                                                                          #
#    This program is free software; you can redistribute it and/or modify  #
#    it under the terms of the GNU General Public License as published by  #
#    the Free Software Foundation; either version 2 of the License, or     #
#    (at your option) any later version.                                   #
#                                                                          #
#    This program is distributed in the hope that it will be useful,       #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of        #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
#    GNU General Public License for more details.                          #
#                                                                          #
#    You should have received a copy of the GNU General Public License     #
#    along with this program; if not, write to the                         #
#    Free Software Foundation, Inc.,                                       #
#    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             #
############################################################################

import pygame
import os
import snowballgame
import player
from gfx_functions import *
from pygame.locals import *

VERSION = "version 0.1.0"

#menus
MENU_MAIN = 0
MENU_CREDITS = 1
MENU_PLAYERS = 2
MENU_ABOUT = 3

class StartMenu:
    "Start menu for tuxletics"

    def __init__(self, screen,font1,font2,font3):

        self.font1=font1
        self.font2=font2
        self.font3=font3

        self.bg=pygame.Surface(screen.get_size())
        self.bg.convert()
        im,re=load_png("startmenubg.png")
        self.bg.blit(im,re)
        self.screen=screen
        self.screen.blit(self.bg,(0,0))

        self.menu=MENU_MAIN
        self.displayMenu()



    def menuMain(self):

        #quick fix
        im,re=load_png("startmenubg.png")
        self.bg.blit(im,re)
        self.screen.blit(self.bg,(0,0))
        pygame.display.flip()
        
        buttonlist2=[Button("Start Event",1,self.font2,self.font3),Button("About",2,self.font2,self.font3),Button("Credits",3,self.font2,self.font3),Button("Quit",4,self.font2,self.font3)]
        t1=["Select this to","Start a Tuxletics Event"]
        t2=["Read about tuxletics","",""]
        t3=["Look at People Behind", "Tuxletics"]
        t4=["Go Back to the Real World"]
        #self.creditsMenu()
        mainoption = self.displayBasicMenu(buttonlist2,t1,t2,t3,t4,VERSION)
        
        if mainoption == 1:
            self.menu = MENU_PLAYERS
        elif mainoption == 2:
            self.menu = MENU_ABOUT
        elif mainoption == 3:
            self.menu = MENU_CREDITS
        else:
            self.menu = -1

    def menuAbout(self):

        #do mask over the bg so that text is shown more clearly
        mask = pygame.Surface((self.screen.get_width(), self.screen.get_height()),pygame.HWSURFACE, self.screen)
        mask.fill((255,255,255))
        mask.set_alpha(150)
        self.screen.blit(mask,mask.get_rect())
        

        help_file = open('README','r')
        help_text = help_file.readlines()
        line_y = 10      # 20

        for line in help_text:
            txt = self.font1.render(str(line).strip(), 1, (21,65,125))  # 42, 131, 251
            txt_rect = txt.get_rect()
            txt_rect.center = (self.screen.get_rect().centerx, line_y)
            self.screen.blit(txt, txt_rect)
            line_y += 10  # 20

        #write press enter to play again to screen
        line_y += 10   # 20
        htext1 = self.font1.render("Press Any key to go back to menu",1,(21,65,125))   # 42, 131, 251
        htext1_rect = htext1.get_rect()
        htext1_rect.center = (self.screen.get_rect().centerx, line_y)
        self.screen.blit(htext1,htext1_rect)

        pygame.display.flip()

        while True:
            for event in pygame.event.get():
                if event.type == pygame.KEYDOWN:
                    self.menu = MENU_MAIN
                    return        

    def menuPlayers(self):

        buttonlist2=[Button("One",1,self.font2,self.font3),Button("Two",2,self.font2,self.font3),Button("Three",3,self.font2,self.font3),Button("Four",4,self.font2,self.font3)]
        t1=["Play alone"]
        t2=["Two player game","Not implemented yet"]
        t3=["Three player game","Not implemented yet"]
        t4=["Four player game", "Not implemented yet"]
        
        numberofplayers = self.displayBasicMenu(buttonlist2,t1,t2,t3,t4,"Select Number of Players")
        
        if numberofplayers == 0 or numberofplayers == 5:
            self.menu = MENU_MAIN
            return
        elif numberofplayers != 1:#if one chooses anything but 1 player will come back to this manu
            self.menu = MENU_PLAYERS
            return
        else:
            players=self.displayPlayerSelect(numberofplayers)
        
            self.game = snowballgame.SnowBallThrowGame(self.screen,players)
            self.game.play()
            self.menu = MENU_MAIN

    def menuCredits(self):
        buttonlist2=[Button("Petteri Klemola",1,self.font2,self.font3),Button("Mauno Haukila",2,self.font2,self.font3),Button("Tero Kuusela",3,self.font2,self.font3),Button("Stefan B.",4,self.font2,self.font3)]
        t1=["Game Design and Programming","Send Sedatives and Virgins to:","pzq@users.sourceforge.net"]
        t2=["Graphix and Other Art","Contact:","mauno@users.sourceforge.net"]
        t3=["Producer, Design and Programming","Contact:","teroajk@users.sourceforge.net"]
        t4=["Design and Programming","Contact:","stefankjb@yahoo.com"]
        #self.creditsMenu()
        creditsoption = self.displayBasicMenu(buttonlist2,t1,t2,t3,t4,"Credits")

        self.menu = MENU_MAIN

    def displayMenu(self):
        while self.menu >= 0:
            if self.menu == MENU_MAIN:
                self.menuMain()
            elif self.menu == MENU_CREDITS:
                self.menuCredits()
            elif self.menu == MENU_PLAYERS:
                self.menuPlayers()
            elif self.menu == MENU_ABOUT:
                self.menuAbout()
            elif self.menu == -1:
                "QUIT GAME"
                break
            else:
                print "ERROR HAS OCCURED" 


    def displayPlayerSelect(self,numplayers):


        #hint texts for button 1
        b1txt=["Type in player names","Names can have only Three letters","When ready push Enter for next", "player"]
        n=1
        ht1 = pygame.sprite.RenderUpdates()
        for x in b1txt:
            h = HintText(x,n,self.font1)
            ht1.add(h)
            n=n+1

        xx = 0

        players=[]
        

        while xx < numplayers:
            
            #init clock
            clock = pygame.time.Clock()
            
            name=""

            buttons = pygame.sprite.RenderUpdates()

            buttons.add(ht1)
            #default selected button is the first button
            selected_button=0

            next_screen=0


            headline = Button("Player names",0.25,self.font2,self.font3)
            #headline.select()

            buttons.add(headline)

            enterpushed=0

            while enterpushed == 0:

                nametxt= Button(name,xx+1,self.font2,self.font3)
                nametxt.select()
                buttons.add(nametxt)
                       
                #limit the fps to 60
                clock.tick(60)
        
                #grab some events if we are not going to next screen
                if next_screen==0:
                    for event in pygame.event.get():
                        if event.type==QUIT:
                            return 0
                        elif event.type==KEYDOWN:
                            if pygame.key.name(event.key) in list('abcdefghijklmnopqrstuvwxyz') and len(name) < 3:
                                name += pygame.key.name(event.key)
                            elif event.key == K_UP:
                                 name = "abc"  
                            elif event.key == K_DOWN:
                                 name = "kkk"
                            elif event.key == K_LEFT:
                                 name = "nnn"
                            elif event.key == K_RIGHT:
                                 name = "jjj"                               
                            elif event.key == K_BACKSPACE or event.key == K_DELETE:
                                name = name[:-1]
                            elif event.key == K_RETURN:
                                if len(name) != 0:
                                    if len(name) == 1:
                                        name += "  "
                                    elif len(name) == 2:
                                        name += " "
                                    players.append(player.Player(name))
                                    enterpushed=1

                        
                buttons.clear(self.screen,self.bg)
                pygame.display.update(buttons.draw(self.screen))

                buttons.remove(nametxt)

            xx=xx+1

        return players
    

    def displayBasicMenu(self,bt,b1txt,b2txt,b3txt,b4txt,text):

        #init clock
        clock = pygame.time.Clock()

        #buttons here
        buttons = pygame.sprite.RenderUpdates()
        buttonlist = bt


        #hint texts for button 1
        n=1
        ht1 = pygame.sprite.RenderUpdates()
        for x in b1txt:
            h = HintText(x,n,self.font1)
            ht1.add(h)
            n=n+1
        #hint texts for button 2
        n=1
        ht2 = pygame.sprite.RenderUpdates()
        for x in b2txt:
            h = HintText(x,n,self.font1)
            ht2.add(h)
            n=n+1
        #hint texts for button 3
        n=1
        ht3 = pygame.sprite.RenderUpdates()
        for x in b3txt:
            h = HintText(x,n,self.font1)
            ht3.add(h)
            n=n+1
        #hint texts for button 4
        n=1
        ht4 = pygame.sprite.RenderUpdates()
        for x in b4txt:
            h = HintText(x,n,self.font1)
            ht4.add(h)
            n=n+1

        #add all the buttons to buttons sprite group
        for x in buttonlist:
            buttons.add(x)

        #default selected button is the first button
        selected_button=0

        next_screen=0


        headline = Button(text,0.25,self.font2,self.font3)
        #headline.select()

        buttons.add(headline)

        while 1 and len(buttonlist)!=0:

            #limit the fps to 60
            clock.tick(60)

            #grab some events if we are not going to next sreen
            if next_screen==0:
                for event in pygame.event.get():
                    if event.type==QUIT:
                        return 0
                    elif event.type==KEYDOWN:
                        if event.key==K_DOWN:
                            if selected_button < len(buttonlist)-1:
                                selected_button=selected_button+1
                        elif event.key==K_UP:
                            if selected_button > 0:
                                selected_button=selected_button-1
                        elif event.key==K_RETURN:
                            next_screen=selected_button+1
                        elif event.key==K_ESCAPE:
                            return 5

            #if we are still in this menu screen
            if next_screen == 0:
                #unselect all buttons
                for x in buttonlist:
                    x.unSelect()
                #select seleacted button
                buttonlist[selected_button].select()

                #we know whois name is in what position so we can draw some text to screen about that person
                if selected_button == 0:
                    buttons.add(ht1)
                    buttons.remove(ht2)
                    buttons.remove(ht3)
                    buttons.remove(ht4)
                elif selected_button == 1:
                    buttons.add(ht2)
                    buttons.remove(ht1)
                    buttons.remove(ht3)
                    buttons.remove(ht4)
                elif selected_button == 2:
                    buttons.add(ht3)
                    buttons.remove(ht1)
                    buttons.remove(ht2)
                    buttons.remove(ht4)
                elif selected_button == 3:
                    buttons.add(ht4)
                    buttons.remove(ht1)
                    buttons.remove(ht2)
                    buttons.remove(ht3)

            #we are going to next menu screen
            elif next_screen!=0:

                buttons.remove(ht1)
                buttons.remove(ht2)
                buttons.remove(ht3)
                buttons.remove(ht4)

                for x in buttonlist:
                    x.move()
                    if x.xpos<-50:   # 100
                        buttonlist.remove(x)
                        x.kill()

            #update button on the screen
            buttons.clear(self.screen,self.bg)
            pygame.display.update(buttons.draw(self.screen))



        
        #clear objects
        for x in buttonlist:
            buttonlist.remove(x)
            x.kill()
        headline.kill()
        buttons.clear(self.screen,self.bg)
        pygame.display.update(buttons.draw(self.screen))

        return next_screen

class HintText(pygame.sprite.Sprite):
    """Some text about the developer"""
    def __init__(self, text,linenumber,font):
        pygame.sprite.Sprite.__init__(self)

        self.image = font.render(text,1,(21,65,125))    # 42,131,251

        self.rect = self.image.get_rect()
        self.rect.topleft=175,0+(10*linenumber)  # 350, 20


class InputBox(pygame.sprite.Sprite):
    "input box class for start menu of tuxletics"
    def __init__(self,text,font2,font3):
        pygame.sprite.Sprite.__init__(self)

        self.image=font3.render(text,1,(21,65,125))  # 42,131,251
        self.rect=self.image.get_rect()

        self.rect.topleft=35,80   # 70, 160

class Button(pygame.sprite.Sprite):
    "button class for start menu of tuxletics"
    def __init__(self,text,number,font2,font3):
        pygame.sprite.Sprite.__init__(self)

	self.image_orginal=font2.render(text,1,(71,94,125))  # 143,189,251
        self.rect_orginal=self.image_orginal.get_rect()
        self.image_selected=font3.render(text, 1,(21,65,125))  # 42,131,251
        self.rect_selected=self.image_selected.get_rect()

        self.image=self.image_orginal
        self.rect=self.rect_orginal
        self.position=number
        self.xpos=35          # 70
        self.rect.topleft=self.xpos, 60+self.position*30     # 120, 60

    def select(self):
        self.image=self.image_selected
    def unSelect(self):
        self.image=self.image_orginal
    def move(self):
        self.xpos=self.xpos-5*self.position
        self.rect.topleft=self.xpos, 60+self.position*30   # 120, 60
